==> waterfall-protocol-1.18-R0.1-20220425.165558-30-sources.jar: net/md_5/bungee/protocol/packet/EncryptionResponse.java <==
package net.md_5.bungee.protocol.packet;

import io.netty.buffer.ByteBuf;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import net.md_5.bungee.protocol.AbstractPacketHandler;
import net.md_5.bungee.protocol.DefinedPacket;
import net.md_5.bungee.protocol.ProtocolConstants;

@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class EncryptionResponse extends DefinedPacket
{

    private byte[] sharedSecret;
    private byte[] verifyToken;

    @Override
    public void read(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion)
    {
        sharedSecret = readArray( buf, 128 );
        verifyToken = readArray( buf, 128 );
    }

    @Override
    public void write(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion)
    {
        writeArray( sharedSecret, buf );
        writeArray( verifyToken, buf );
    }

    @Override
    public void handle(AbstractPacketHandler handler) throws Exception
    {
        handler.handle( this );
    }

    // Waterfall start: Additional DoS mitigations, courtesy of Velocity
    public int expectedMaxLength(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion) {
        // It turns out these come out to the same length, whether we're talking >=1.8 or not.
        // The length prefix always winds up being 2 bytes.
        return 260;
    }

    public int expectedMinLength(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion) {
        return expectedMaxLength(buf, direction, protocolVersion);
    }
    // Waterfall end
}

==> waterfall-protocol-1.19-R0.1-20221114.191829-19-sources.jar: net/md_5/bungee/protocol/packet/EncryptionResponse.java <==
package net.md_5.bungee.protocol.packet;

import io.netty.buffer.ByteBuf;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import net.md_5.bungee.protocol.AbstractPacketHandler;
import net.md_5.bungee.protocol.DefinedPacket;
import net.md_5.bungee.protocol.ProtocolConstants;

@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class EncryptionResponse extends DefinedPacket
{

    private byte[] sharedSecret;
    private byte[] verifyToken;
    private EncryptionData encryptionData;

    public void read(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion)
    {
        sharedSecret = readArray( buf, 128 );
        if ( protocolVersion < ProtocolConstants.MINECRAFT_1_19 || buf.readBoolean() )
        {
            verifyToken = readArray( buf, 128 );
        } else
        {
            encryptionData = new EncryptionData( buf.readLong(), readArray( buf ) );
        }
    }

    @Override
    public void write(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion)
    {
        writeArray( sharedSecret, buf );
        if ( verifyToken != null )
        {
            if ( protocolVersion >= ProtocolConstants.MINECRAFT_1_19 )
            {
                buf.writeBoolean( true );
            }
            writeArray( verifyToken, buf );
        } else
        {
            buf.writeLong( encryptionData.getSalt() );
            writeArray( encryptionData.getSignature(), buf );
        }
    }

    @Override
    public void handle(AbstractPacketHandler handler) throws Exception
    {
        handler.handle( this );
    }

    @Data
    public static class EncryptionData
    {

        private final long salt;
        private final byte[] signature;
    }

    // Waterfall start: Additional DoS mitigations, courtesy of Velocity
    public int expectedMaxLength(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion) {
        // It turns out these come out to the same length, whether we're talking >=1.8 or not.
        // The length prefix always winds up being 2 bytes.
        if (protocolVersion >= ProtocolConstants.MINECRAFT_1_19) return -1;
        return 260;
    }

    public int expectedMinLength(ByteBuf buf, ProtocolConstants.Direction direction, int protocolVersion) {
        return expectedMaxLength(buf, direction, protocolVersion);
    }
    // Waterfall end
}
